import importlib
import importlib.util
import sys
from pathlib import Path
import sysconfig

EXT_SUFFIX = sysconfig.get_config_var("EXT_SUFFIX")

target_dll = Path(__file__).parent / f'libpymatio{EXT_SUFFIX}'
try:

    spec = importlib.util.spec_from_file_location("libpymatio", target_dll)
    libpymatio = importlib.util.module_from_spec(spec)
    sys.modules["libpymatio"] = libpymatio
    spec.loader.exec_module(libpymatio)
except Exception as e:
    import traceback
    traceback.print_exc()
    raise

from libpymatio import get_library_version
from libpymatio import *



import os
import shutil
import subprocess
import sysconfig
from pathlib import Path
import logging

from setuptools import Extension
from setuptools.command.build_ext import build_ext


class XmakeBuildExt(build_ext):
    def run(self):
        self.xmake_build()
        self.copy_output_file()

    def xmake_build(self):
        platform = sysconfig.get_platform()
        xmake_archs = {
            'win-amd64': 'x64',
            'win32': 'x86',
            'linux-x86_64': 'x86_64',
            'linux-i686': 'i386',
            'darwin-x86_64': 'x86_64',
            'darwin-arm64': 'arm64',
        }
        curr_arch = xmake_archs.get(platform)
        if curr_arch is None:
            raise Exception(f'Unsupported platform: {platform}, allowed: {xmake_archs}')
        logging.debug(f"{curr_arch=} {platform=}")
        # 检查xmake是否存在
        if not shutil.which("xmake"):
            raise EnvironmentError("xmake is not installed or not found in PATH. \nTo install xmake, please refer to https://xmake.io/#/guide/installation")
        subprocess.run(["xmake", "config", "-c", "-a", curr_arch, "-v", "-D", '-y'])
        subprocess.run(["xmake", "build", '-y', '-v', '-D'])


    def copy_output_file(self):
        ext_suffix = sysconfig.get_config_var("EXT_SUFFIX")
        built_files = list(Path('build').glob(f'**/libpymatio{ext_suffix}'))
        if not built_files:
            raise FileNotFoundError(f"No libpymatio file found after xmake build @ {Path('build').absolute()}")
        built_file = str(built_files[0])
        logging.info(f"Found built library: {built_file} @ {Path('.').absolute()}")
        
        self.mkpath(os.path.join(self.build_lib, 'pymatio'))
        self.copy_file(built_file, os.path.join(self.build_lib, 'pymatio'))
        self.copy_extensions_to_source()


def build(setup_kwargs):
    print(setup_kwargs)
    setup_kwargs.update({
        'ext_modules': [Extension('pymatio.libpymatio', sources=[])],
        'cmdclass': {'build_ext': XmakeBuildExt},
        'package_data': {
            'pymatio': [f'libpymatio{sysconfig.get_config_var("EXT_SUFFIX")}'],
        },
    })

    print(setup_kwargs)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['pymatio', 'src']

package_data = \
{'': ['*']}

modules = \
['xmake']
setup_kwargs = {
    'name': 'pymatio',
    'version': '0.1.0',
    'description': '',
    'long_description': '[中文](./README.md) [English](./README_en.md)\n\n# 背景\n\nPython 中始终没有一个一站式读取 mat 文件的库，mat5 总是依赖 scipy.io, mat7.3 总是依赖 h5py，h5py 直接读取 mat 文件又需要很多手动转换，有一个 mat73 转换，但是核心逻辑是纯 Python 写的，又非常慢。\n\n恰巧 C 中有一个库 [matio](https://github.com/tbeu/matio)，我就想用 pybind11 做一个绑定。\n\n# 路线\n\n- [x] 完成基本函数的绑定\n- [x] 使用 xmake 在 Windows 和 Linux 上编译通过\n- [x] 打包为 whl 文件\n- [x] 添加关于构建成功的基本测试\n- [ ] 添加 cibuildwheel 和 Github Action\n- [ ] 吸取 https://github.com/pybind/scikit_build_example 中的优势\n- [x] 编译扩展时自动处理虚拟环境\n- [ ] 添加更 Pythonic 的调用接口\n- [ ] 添加 benchmark\n- [ ] 导入 scio 的测试和 mat73 的测试\n\n# 使用\n\n当前还没有打包到 whl，如需试用需安装 [xmake](https://github.com/xmake-io/xmake/)，这是一个基于 lua 的构建系统，非常轻量。\n\n```bash\ngit clone https://github.com/myuanz/pymatio\nxmake\n```\n\n之后即可在`build`目录下见对应平台的 Python 扩展，hdf5 和 matio 之依赖会自动处理。\n\n# 样例\n\n```python\nimport pymatio as pm\n\nprint(pm.get_library_version())\nprint(pm.log_init(\'pymatio\'))\nprint(pm.set_debug(1))\npm.critical("abcdefg%d,%d\\n" % (234, 456,))\nmat = pm.create_ver(\'test.mat\', None, pm.MatFt.MAT73)\n\nvar1 = pm.var_create(\'var1\', pm.MatioClasses.DOUBLE, pm.MatioTypes.DOUBLE, 2, (2, 3,), (1, 2, 3, 4, 5, 6,), 0)\npm.var_write(mat, var1, pm.MatioCompression.NONE)\npm.var_free(var1)\nprint(mat.filename, mat.version, mat.fp, mat.header, mat.byte_swap, mat.mode, mat.bof, mat.next_index, mat.num_datasets, mat.refs_id, mat.dir)\n```\n\n输出：\n\n```\n(1, 5, 26)\n0\nNone\n-E- abcdefg234,456\n: abcdefg234,456\n\ntest.mat 512 <capsule object NULL at 0x7f85197b3fc0> MATLAB 7.3 MAT-file, Platform: x86_64-pc-Linux, Created by: libmatio v1.5.26 on Tue Dec 26 15:49:00 2023\n HDF5 sche 0 MatAcc.RDWR 128 0 1 -1 []\n ```',
    'author': 'zhengmy',
    'author_email': 'zhengmy@ion.ac.cn',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'py_modules': modules,
    'python_requires': '>=3.9,<3.13',
}
from build_arg import *
build(setup_kwargs)

setup(**setup_kwargs)
